package top.daxian1218.dust.license.issue;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.log.LoggerHelper;
import top.daxian1218.dust.commons.util.Base64Util;
import top.daxian1218.dust.license.LicenseConfig;
import top.daxian1218.dust.license.LicenseException;
import top.daxian1218.dust.license.LicenseInfo;
import top.daxian1218.dust.license.file.FileKeyPair;
import top.daxian1218.dust.license.util.LicenseFileUtil;
import top.daxian1218.dust.license.util.RSAUtil;

import java.io.InputStream;
import java.util.Map;

/**
 * License发行服务
 * @author daxian
 */
public class LicenseIssueService {
    /**
     * 发行签名程序以及签名文件
     * @param steam
     * @param fileName
     * @return
     */
    public FileKeyPair getFileKeyPair(InputStream steam, String fileName) {
        FileKeyPair fileKeyPair = new FileKeyPair();

        String hashcode = LicenseFileUtil.hash(steam);
        fileKeyPair.setHash(hashcode);

        Map<String, String> keyPair = RSAUtil.generateKeyPair();
        fileKeyPair.setPrivateKey(keyPair.get("privateKey"));
        fileKeyPair.setPublicKey(keyPair.get("publicKey"));

        JSONObject jo = new JSONObject();
        jo.put("cpuCheck", LicenseConfig.INSTANCE.isCpuCheck());
        jo.put("mainboardCheck", LicenseConfig.INSTANCE.isMainboardCheck());
        jo.put("hostCheck", LicenseConfig.INSTANCE.isHostNameCheck());
        jo.put("macCheck", LicenseConfig.INSTANCE.isMacCheck());
        jo.put("name", fileName);
        jo.put("publicKey", keyPair.get("publicKey"));
        jo.put("sign", RSAUtil.sign(hashcode, keyPair.get("privateKey")));
        fileKeyPair.setSignedContext(Base64Util.encodeEx(jo.toJSONString()));
        return fileKeyPair;
    }

    /**
     * 获取设备指纹
     * @param torrent
     * @param privateKey
     * @return
     */
    public String getDeviceFingerPrintOfTorrent(String torrent, String privateKey) {
        if (StringUtils.isEmpty(torrent)) return null;

        String content = Base64Util.decodeEx(torrent);
        try {
            return RSAUtil.decrypt(content, privateKey);

        } catch (LicenseException e) {
            LoggerHelper.error(e);
            return null;
        }
    }

    /**
     * 生成License内容
     * @param body
     * @param expiryTime
     * @param fingerPrint
     * @param trial
     * @return
     */
    public String generateLicense(String body, long expiryTime, String fingerPrint, boolean trial) {
        if (StringUtils.isEmpty(LicenseConfig.INSTANCE.getPrivateKey())) {
            throw new LicenseException("生成License失败, 需要核心库私钥放置于发行端", null);
        }
        LicenseInfo licenseInfo = new LicenseInfo();
        licenseInfo.setSubject(LicenseConfig.INSTANCE.getSubject());
        licenseInfo.setIssuedTime(System.currentTimeMillis());
        licenseInfo.setExpiryTime(expiryTime);
        licenseInfo.setDeviceFingerPrint(fingerPrint);
        licenseInfo.setTrial(trial);
        licenseInfo.setBody(body);

        String license = JSON.toJSONString(licenseInfo);

        String signKey = RSAUtil.sign(license, LicenseConfig.INSTANCE.getPrivateKey());
        return Base64Util.encodeEx(license + "." + signKey);
    }
}
